unit MathTest;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, MathParser, StdCtrls, Math, ComCtrls, ExtCtrls;

type
  Tfmain = class(TForm)
    PageControl: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    pbfunction: TPaintBox;
    Panel1: TPanel;
    Label3: TLabel;
    efunction: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    eterm: TEdit;
    Button1: TButton;
    mres: TMemo;
    Button2: TButton;
    procedure Button1Click(Sender: TObject);
    procedure pbfunctionPaint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure pbfunctionMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    FFunctionVisibleRange: integer;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  fmain: Tfmain;

  ACalcParser: TMathParser;
  
  AFuncParser: TMathParser;
  AFuncTree: PTTermTreeNode;

implementation

{$R *.dfm}



procedure Tfmain.Button1Click(Sender: TObject);
var CalcResult: Extended;
    FTree: PTTermTreeNode;
begin
  try
    FTree:=ACalcParser.ParseTerm(eterm.Text);
    CalcResult:=ACalcParser.CalcTree(FTree);
  except
    on e: EMathParserException do Begin
      mres.Lines.Insert(0,'(ERROR) '+e.ErrorDescription+#13#10);
      eterm.SetFocus;
      eterm.SelStart:=e.ErrorPos-1;
      eterm.SelLength:=e.ErrorLength;
    end;
  end;

  if ACalcParser.ParseError=mpeNone then
    mres.Lines.Insert(0,ACalcParser.FormatTerm(FTree)+' = '+FloatToStr(CalcResult)+#13#10);

  ACalcParser.FreeLastTermTree;
end;


procedure Tfmain.pbfunctionPaint(Sender: TObject);
var mx, my, wx, wy: integer;
    i: integer;
    errorinlast: boolean;
const border = 10;
begin
  with pbfunction do Begin
    //calc center
    mx:=(Width - border*2) div 2;
    my:=(Height - border*2) div 2;
    //calc size
    wx:=mx div FFunctionVisibleRange;
    wy:=my div FFunctionVisibleRange;

    with Canvas do Begin
      //Reset
      brush.Color:=clWhite;
      FillRect(ClientRect);

      //paint axes
      pen.Color:=clBlack;
      MoveTo(border,my+border); LineTo(pbfunction.Width-border,my+border);
      MoveTo(border+mx,border); LineTo(border+mx,pbfunction.Height-border);
      for i:=-FFunctionVisibleRange to FFunctionVisibleRange do Begin
        if (i=0) or
           ((FFunctionVisibleRange>=10) and (i mod (FFunctionVisibleRange div 10) <> 0)) then continue;
        TextOut(border + mx + (i * wx) - (TextWidth(inttostr(i)) div 2),
                border + my + 2,
                inttostr(i));
        MoveTo(border + mx + (i * wx), border + my - 2);
        LineTo(border + mx + (i * wx), border + my + 2);
      end;
      for i:=-FFunctionVisibleRange to FFunctionVisibleRange do Begin
        if (i=0) or
           ((FFunctionVisibleRange>=10) and (i mod (FFunctionVisibleRange div 10) <> 0)) then continue;
        TextOut(border + mx + 4,
                border + my - (i * wy) - (TextHeight(inttostr(i)) div 2),
                inttostr(i));
        MoveTo(border + mx + 2, border + my + (i * wy));
        LineTo(border + mx - 2, border + my + (i * wy));
      end;

      if AFuncTree=nil then exit;  //exit if no tree

      try
        pen.Color:=clBlue;

        try
          AFuncParser.ConstantValues['x']:=(FFunctionVisibleRange*2)*100;
          MoveTo(trunc(border + mx + int(-(FFunctionVisibleRange*2) * wx)),
                 trunc(border + my + int(AFuncParser.CalcTree(AFuncTree) * wy)));
        except
          errorinlast:=true;
        end;

        errorinlast:=false;
        for i:=-(FFunctionVisibleRange*2)*100 to (FFunctionVisibleRange*2)*100 do Begin
          try
            AFuncParser.ConstantValues['x']:=i / 100;
            if not errorinlast then
              LineTo(trunc(border + mx + (i / 100 * wx)),
                     trunc(border + my - (AFuncParser.CalcTree(AFuncTree) * wy)))
            else Begin
              MoveTo(trunc(border + mx + (i / 100 * wx)),
                     trunc(border + my - (AFuncParser.CalcTree(AFuncTree) * wy)));
              errorinlast:=false;
            end;
          except errorinlast:=true; end;
        end;
      except
        on e: EMathParserException do Begin
          AFuncParser.FreeTermTree(AFuncTree);
          ShowMessage('(ERROR) '+e.ErrorDescription);
          efunction.SetFocus;
          efunction.SelStart:=e.ErrorPos-1;
          efunction.SelLength:=e.ErrorLength;
          exit;
        end;
      end;
    end;
  end;
end;

procedure Tfmain.FormCreate(Sender: TObject);
begin
  fmain.PageControl.ActivePageIndex:=0;
  FFunctionVisibleRange:=10;

  ACalcParser:=TMathParser.Create;
  ACalcParser.RegisterExtendedOperators;
  ACalcParser.RegisterConversionOperators;

  AFuncParser:=TMathParser.Create;
  AFuncParser.RegisterExtendedOperators;
  AFuncParser.RegisterConversionOperators;
  AFuncParser.AddConstant('x',0);
end;

procedure Tfmain.FormDestroy(Sender: TObject);
begin
 ACalcParser.FreeLastTermTree;
 ACalcParser.Free;
 AFuncParser.FreeTermTree(AFuncTree);
 AFuncParser.FreeLastTermTree;
 AFuncParser.Free;
end;

procedure Tfmain.Button2Click(Sender: TObject);
begin
  try
    AFuncTree:=AFuncParser.ParseTerm(efunction.Text);
  except
    on e: EMathParserException do Begin
      ShowMessage('(ERROR) '+e.ErrorDescription);
      efunction.SetFocus;
      efunction.SelStart:=e.ErrorPos-1;
      efunction.SelLength:=e.ErrorLength;
    end;
  end;

  if AFuncParser.ParseError=mpeNone then pbfunction.Repaint
  else AFuncParser.FreeTermTree(AFuncTree);
end;

procedure Tfmain.pbfunctionMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 if BUtton=mbLeft then Inc(FFunctionVisibleRange)
 else if FFunctionVisibleRange>1 then Dec(FFunctionVisibleRange);

 pbfunction.Repaint;
end;

end.
